/**
 * 
 */
package bricktest;

import java.util.Random;

import de.grosenick.brickapi.NetworkThread;
import de.grosenick.brickapi.commands.MoveServo;
import de.grosenick.brickapi.commands.SetRelay;

/**
 *
 */
public class ServoThread extends Thread
{
    private NetworkThread  nt;
    private boolean finish;

    /**
     * 
     */
    public ServoThread(NetworkThread nt)
    {
        super("ServoThread");
        this.nt = nt;
    }

    @Override
    public void run()
    {
        // Bewege das Servo an PIN 2 von 1..7, Servoweg +/- 45°
        MoveServo command1 = new MoveServo(2, 45, true);
        // und synchron Servo 5, aber invertiert
        command1.setServo2(5, true);

        MoveServo command2 = new MoveServo(1, 45, true);
        command2.setServo2(6, true);
        int direction = 1;
        
        // 1. Switch in der Sequenz 0..3
        SetRelay relay1 = new SetRelay(null, 0);
        relay1.switchOn();
        
        Random rand = new Random();
        long start = System.currentTimeMillis();
        long last  = start;
        rand.setSeed(start);

        // try to create about 120 commands / second
        while (!finish)
        {
            try {
                if (command1.isIdle())
                    nt.enqueueCommand(command1);
                if (command1.isIdle())
                    nt.enqueueCommand(command2);
                
                long act = System.currentTimeMillis();
                if (act - last >= 1654) {
                    relay1.setDuration(200);
                    // is the command is in the queue for more than 1,7 seconds, 
                    // it will be put into the queue again here - and executed twice ...
                    nt.enqueueCommand(relay1);
                    last = act;
                }
                
                Thread.sleep(rand.nextInt(9));
            }
            catch (InterruptedException ie) {
                // ignore
            }

            int percent = command1.getPercentValue() + direction;
            if (direction > 0)
            {
                if (percent >= 100) {
                    percent = 100;
                    direction = -direction;
                }
            }
            else {
                if (percent <= -100) {
                    percent = -100;
                    direction = -direction;
                }
            }
            command1.rotateTo(percent);
            command2.rotateTo(percent);
        }
        command1.rotateTo(0);
        command2.rotateTo(0);
        nt.enqueueCommand(command1);
        nt.enqueueCommand(command2);
        
        double ms = (System.currentTimeMillis() - start);
        System.out.println("Send " + (command1.getCallCount() + command2.getCallCount()) + " Commands in " + 
                ms + "ms => " + ((command1.getCallCount() + command2.getCallCount())/ms*1000.0) + " commands/s");
    }
    
    public void doFinish() {
        finish = true;
    }
}
