package de.grosenick.brickapi;

/**
 * This interface describes objects that consume the commands which
 * should be send to the bricks / bricklets.
 * <P>
 * The commands pushed into a queue and typically consumed in another
 * thread.
 * 
 * @author Holger Grosenick
 */
public interface CommandQueue
{

    /**
     * Signals the thread to stop, this is done as soon as possible.
     */
    void doFinish();   
    
    /**
     * Sends a brick-command to the network queue. The command is executed
     * in another thread.
     * @param cmd command to execute.
     */
    void enqueueCommand(Command cmd);
    
    
    /**
     * Returns true if the (other) thread is currently executing a command.
     * @return true of false
     */
    boolean isBusy();
    
}
