
package de.grosenick.brickapi;

import java.util.concurrent.ArrayBlockingQueue;

import de.grosenick.brickapi.commands.DoNothing;


/**
 * Example for an async thread that sends command to the brick stack.
 * 
 * @author Holger Grosenick
 */
public class NetworkThread extends Thread implements CommandQueue
{
    private BrickConnector brickConnector;
    private ArrayBlockingQueue<Command> queue;
    private boolean finish = false;
    private boolean busy   = false;

    /**
     * 
     * @param brickConnector
     */
    public NetworkThread(BrickConnector brickConnector)
    {
        super("NetworkThread");
        this.queue = new ArrayBlockingQueue<Command>(100);
        this.brickConnector = brickConnector;
    }

    @Override
    public void run()
    {
        boolean stop = false;

        while (!stop)
        {
            try {
                busy = false;
                Command item = queue.take();
                busy = true;
                item.execute(brickConnector);

                // close task only if queue is empty
                if (queue.size() == 0)
                    stop = finish;
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
        busy = false;
    }

    /**
     * Signals the thread to stop, this is done as soon as possible.
     */
    @Override
    public void doFinish() {
        finish = true;
        
        // add a command, this wakes up the thread
        enqueueCommand(new DoNothing());
    }
    
    /**
     * Sends a brick-command to the network queue. The command is executed
     * in another thread.
     * @param item command to execute.
     */
    @Override
    public void enqueueCommand(Command item)
    {
        try {
            queue.put(item);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Returns true if the task is currently executing a command.
     * @return true of false
     */
    @Override
    public boolean isBusy() {
        return busy;
    }
}
